from .shallownet import ShallowNet
from .agegendernet import AgeGenderNet
from .alexnet import AlexNet
from .deepergooglenet import DeeperGoogLeNet
from .lenet import LeNet
from .minigooglenet import MiniGoogLeNet
from .minivggnet import MiniVGGNet
from .resnet import ResNet